function [t,cellfractions,molecularconc,G] = HepatRegenGrowth_classification_virtual_patient_analysis()
% Author:       Babita Verma
% Date:         12/02/2017
%
% Description:  Solves the system of equations govenrning liver
%               regeneration with the  parameters from cook et al.,"Systems Analysis of Non-Parenchymal Cell Modulation
%               of Liver Repair across Multiple Regeneration Modes", BMC Systems Biology, 2015;  for
%               human given in Table 1 and classifies
%               each virtual patient based on the extent of recovery.
% 
% Input:    To simulate, give the sample size of virtual patients and initial fraction of remnant liver
%           fraction as input to the function or the code uses the default values of: sample size (nruns) = 100; initial
%           fraction = 0.9.
%
% Output:   Generates the parameter space of metabolic load and cell death
%           sensitivities which shows classification of virtual patients as normal,
%           suppressed, delayed liver failrue and immediate liver failure class.
%
% Returns variables:    t: time in days post resection
%                       cellfractions:  the fraction of quiescent(Q),priming(P), and replicating
%                                       cells(R); matrix whose columns represent Q,P,R and rows denote time
%                                       point
%                       molecularconc:  the concentration of IE genes, GF,ECM,IL6, JAK, STAT3, SOCS3;
%                                       matrix whose columns represent molecular concentration and rows
%                                       time
%                       G:              relative cell mass; Vector
%
% citation: Babita K. Verma, Pushpavanam Subramaniam, Rajanikanth Vadigepalli,
%           "Modeling the dynamics of human liver failure post 
%            liver resection"

%% Main code begins

global eps Nss

%define default sample size of virtual patient
nruns = 50;
%input sample size of virtual patient
user_nruns = input('\nDefault sample size of virtual patients is 100.\nTo change the sample size, type the new value here else press enter: ');
%assigning user defined sample size to nruns
if ~isempty(user_nruns)
    nruns = user_nruns;
end

%define default value of remnant liver fraction
initialFraction=0.9;
%asking user to input initial fraction of remnant liver fraction
user_initialFraction = input('\nDefault fraction of remnant liver mass is 0.9.\nTo change the intial liver fraction, type the new value here else press enter : ');
%assigning user defined initial fraction
if ~isempty(user_initialFraction)
    initialFraction = user_initialFraction;
end

%generate sobol sequence
[seq1,seq2,seq3] = Sobolsample(nruns);

%initial steady state of liver mass
Nss=0.99;
eps=0.01;

% Table 1:  Define model parameters
k = zeros(41,1);
k(1) = initialFraction; %remnant liver fraction
k(2)  = 5.8507;         % Metabolic load (M)
k(3)  = 1.5;            % IL6 production (k_IL6)
k(4)  = 0.9;            % IL6 mRNA degrad (kappa_IL6)
k(5)  = 20000;          % JAK activation rate (V_JAK)
k(6)  = 10000;          % (k^JAK_M)
k(7)  = 0.4;            % JAK degradation rate (kappa_JAK)
k(8)  = 2;              % Conc. of monomeric STAT3 ([proSTAT3])
k(9)  = 750;            % STAT3 activation rate (V_ST3)
k(10)  = 0.4;           % (k^ST3_M)
k(11) = 0.1;            % STAT3 degradation rate(kappa_ST3)
k(12) = 24000;          % SOCS3 activation rate (V_SOCS3)
k(13) = 7e-4;           % (K^SOCS3_M)
k(14) = 0.4;            % SOCS3 degradation (kappa_SOCS3)
k(15) = 0.015;          % SOCS3 inhibition constant (K^SOCS3_l)
k(16) = 250;            % IE response gene expression rate (V_IE)
k(17) = 18;             % (K^IE_M)
k(18) = 5;              % IE gene degrad (kappa_IE)
k(19) = 7;              % ECM degrad by MMPs (k_deg)
k(20) = 33;             % ECM degradation rate (kappa_ECM)
k(21) = 0.113;          % GF production (k_GF)
k(22) = 0.23;           % GF degradation(kappa_GF)
k(23) = 0.06;           % GF uptake/production rate by ECM (k_up)
k(24) = 0.007;          % Q to P conversion rate constant(k_QP)
k(25) = 0.0044;         % P to R conversion rate constant (k_PR)
k(26) = 0.054;          % R to Q conversion rate constant (k_RQ)
k(27) = 0.02;           % specifies length of mitotic cycle (k_prol)
k(28) = 0.1;            % requiescence rate (k_req)
k(29) = 8;              % (theta_req) for requiescence calc
k(30) = 3;              % (beta_req) for requiescence calc
k(31) = 0.1;            % cell death constant (k_cd)
k(32) = 0.032;          % cell death sensitivity (theta_cd)
k(33) = 0.0045;         % cell death sensitivity  (beta_cd)
k(41) = 6.5675e-4;      % growth constant for cell mass (k_G)


%upper and lower bound for the three model controlling parameters: Metabolic load (M) &
%cell death sensitivitiy parameters(theta_cd, beta_cd)
lb=0.1*[k(2) k(32) k(33)];
ub=[2*k(2) 3.125*k(32) 7*k(33)];


%sequence of parameter set using sobol sampling
M_seq=lb(1)+(ub(1)-lb(1)).*seq1;
thetaap_seq=lb(2)+(ub(2)-lb(2)).*seq2;
betaap_seq=lb(end)+(ub(end)-lb(end)).*seq3;

%initialize variables to store failed and sucessful integration
failedintegration=zeros(1,nruns);
l1=1;l2=1;
success=zeros(1,nruns);

iter=[1:length(M_seq)];

%initialize the bounds of time for simulations
tStart = 0;
tEnd=910*24; %day to hour conversion

for j=1:length(iter)
    try
        i(j)=iter(j);
        fprintf('Running virtual patient simulation: patient #%d for resection of %.2f \n',i(j), (1-initialFraction));
        k(2) = M_seq(i(j));
        k(32) = thetaap_seq(i(j));
        k(33) = betaap_seq(i(j));
        % k(32) = k(32)*20.8217/k(2); % Scale the cell death response
        
        % Material and Methods: Mathematical model (Steady-State
        % parameters)
        k(34) = -k(3)*k(2) + k(5)/(1+k(6)) + k(4); % TNF production
        k(35) = -k(5)/(1+k(6)) + k(7); % JAK production
        k(36) = -k(9)*k(8)^2/(k(8)^2+k(10)*(1+1/k(15))) + ...
            k(16)/(1+k(17)) + k(12)/(1+k(13)) + k(11); % STAT3 production
        k(37) = -k(12)/(1+k(13)) + k(14); % SOCS production
        k(38) = -k(16)/(1+k(17)) + k(18); % IE gene production
        k(39) = k(19) + k(20); % ECM production
        k(40) = -k(21)*k(2)/(Nss+eps) + k(23) + k(22); % GF production
        
        
        % Set initial hepatocyte states, cell = (Q,P,R)
        % and concentrations, conc = [IE,GF,ECM,TNF,JAK,STAT3,SOCS3]
        cellfractions = [k(1),0,0];
        molecularconc = ones(1,7);
        G = 1;
        x0 = [cellfractions,molecularconc,G]';
        
        disp('Inside ode solver');
        % Call ODE solver
        [t,x] = ode15s(@(t,x)LiverRegenODE(t,x,k,x0), [tStart:0.1:tEnd], x0);
        disp('Outside ode solver');
        
        % Organize results
        
        % Remove negative values not used in calculations
        x(x(:,1) < 0,1) = 0;
        x(x(:,2) < 0,2) = 0;
        x(x(:,3) < 0,3) = 0;
        x(x(:,11) < 0,11) = 0;
        
        
        cell_pp(:,1:3,l1)=x(:,1:3);
        conc_pp(:,1:7,l1)=x(:,4:10);
        G_pp(:,:,l1)=x(:,11);
        
        
        % calculating liver fraction
        Nf(:,1,l1)=x(:,1)+x(:,11).*(x(:,2)+x(:,3));
        
        %storing successful iteration info
        success(l1)=i(j);
        l1=l1+1;
        
        
    catch
        %storing failed iteration info
        failedintegration(l2)=i(j);
        fprintf('error occurred at : %d\n',i(j))
        l2=l2+1;
        
    end
    
end

%changing time from hour to days
t=t/24;

%storing the address of success and failed integration
successad(1,1:(l1-1))=success(1,1:(l1-1));
failureintegrationad(1,1:(l2-1))=failedintegration(1,1:(l2-1));

%calculating the case of liver failure
livermass2_5yr=Nf(end,1,:);
address_liverfailure=find(livermass2_5yr<0.1);

%iter
%finding address of recovered patient
liverfail_paraset_ad=successad(address_liverfailure);
common=intersect(liverfail_paraset_ad,successad);
liver_recovered=setxor(successad,common);

%l1
%storing all the recovered patients information
address_liver_recovered=zeros(1,length(liver_recovered));
for j=1:length(liver_recovered)
    address_liver_recovered(j)=find(successad==liver_recovered(j));
end


%finding the address of different recovered category patients by excluding liver failure
%case
finallivermass=Nf(end,1,address_liver_recovered);
address_suppressed=address_liver_recovered(find(finallivermass<0.9));
address_normal=address_liver_recovered(find((finallivermass>=0.9)&(finallivermass<=1.1)));
address_enhanced=address_liver_recovered(find((finallivermass>1.1)));

%finding the address of delayed and immediate liver failure
t5days=find(t==5,1);
N5day_LF=Nf(t5days,1,address_liverfailure);
iLF=find(N5day_LF<0.1);
dLF=find(N5day_LF>0.1);
address_iLF=address_liverfailure(iLF);
address_dLF=address_liverfailure(dLF);

%finallivermass in 2D matrix
Nf2d(:,1:(l1-1))=Nf(:,1,1:(l1-1));
clear Nf;

%calculation of cell death profile for delayed liver failure class
M_seq_dLF=M_seq(address_dLF);
thetaap_seq_dLF=thetaap_seq(address_dLF);
betaap_seq_dLF=betaap_seq(address_dLF);

%Nt is fraction of liver changing with time, sigfn_dLF is the cell death
%function for delayed liver failure class
Nt=(0:0.01:1)';
sigfn_dLF=zeros(length(Nt),length(M_seq_dLF));

%evaluating cell seath function value
for i=1:length(M_seq_dLF)
sigfn_dLF(:,i)=0.5*(1+tanh((thetaap_seq_dLF(i)-(Nt./M_seq_dLF(i)))/(betaap_seq_dLF(i))));
end

% save('variables_nom_BMC_sobol3para_11M_0_0_3betaap_0_0_1thetaap_nruns3000_PHx_0.1_3Mnom.mat','-v7.3');

%3D parameter space for all 4 classification
figure
set(gca,'FontSize',12);
h1(1)=plot3(M_seq(successad(address_normal)),thetaap_seq(successad(address_normal)),betaap_seq(successad(address_normal)),'o','MarkerFaceColor',[0 153/255 0],'MarkerEdgeColor',[0 153/255 0],'MarkerSize',4);
hold on
h1(2)=plot3(M_seq(successad(address_suppressed)),thetaap_seq(successad(address_suppressed)),betaap_seq(successad(address_suppressed)),'bo','MarkerFaceColor','b','MarkerSize',4);
h1(3)=plot3(M_seq(successad(address_iLF)),thetaap_seq(successad(address_iLF)),betaap_seq(successad(address_iLF)),'ms','MarkerFaceColor','m','MarkerSize',4);
h1(4)=plot3(M_seq(successad(address_dLF)),thetaap_seq(successad(address_dLF)),betaap_seq(successad(address_dLF)),'c^','MarkerFaceColor','c','MarkerSize',4);
% legend(h1,{'Normal growth','Suppressed growth','Immediate liver failure','Delayed liver failure'});
xlabel('Metabolic load (M)');
ylabel('Cell death sensitivity (\theta_{ap})','rotation',-22);
zlabel('Cell death sensitivity (\beta_{ap})');
% xlabel('Metabolic load (M)','rotation',5);
% ylabel('Cell death sensitivity (\theta_{ap})','rotation',-69);
% zlabel('Cell death sensitivity (\beta_{ap})');
set(gca,'Ydir','reverse')
title(['Resection ',num2str((1-initialFraction)*100),' %']);
% campos([-10.00348,2.41485,0.28223]);
box on


%regeneratin profile of recovered classes : Normal , Suppressed
figure;hold on;
set(gca,'FontSize',12);
for i=1:length(address_normal)
    h2(1)=plot(t,cell_pp(:,1,address_normal(i))+G_pp(:,1,address_normal(i)).*(cell_pp(:,2,address_normal(i))+cell_pp(:,3,address_normal(i))),'Color',[0 153/255 0],'linewidth',1);
end
for i=1:length(address_suppressed)
    h2(2)= plot(t,cell_pp(:,1,address_suppressed(i))+G_pp(:,1,address_suppressed(i)).*(cell_pp(:,2,address_suppressed(i))+cell_pp(:,3,address_suppressed(i))),'b','linewidth',1);
end
xlabel('Time post resection (days)');
ylabel('Fraction of liver')
ylim([-0.02 1]);
xlim([-10 180]);
legend(h2,{'Normal','Suppressed'},'location','best')
title('Regeneration profile of normal & suppressed class')
box on


%Temporal profile for fraction of liver for liver failure class
figure;hold on;
set(gca,'FontSize',12);
for i=1:length(address_dLF)
    h3(1)= plot(t,cell_pp(:,1,address_dLF(i))+G_pp(:,1,address_dLF(i)).*(cell_pp(:,2,address_dLF(i))+cell_pp(:,3,address_dLF(i))),'c','linewidth',1);
end
for i=1:length(address_iLF)
    h3(2)=plot(t,cell_pp(:,1,address_iLF(i))+G_pp(:,1,address_iLF(i)).*(cell_pp(:,2,address_iLF(i))+cell_pp(:,3,address_iLF(i))),'m','linewidth',1);
end
hold off;box on;
xlabel('Time post resection (days)');
ylabel('Fraction of liver')
ylim([-0.03 1]);
xlim([-10 180]);
legend(h3,{'Delayed failure','Immediate failure'},'location','best')
title('Regeneration profile of liver failure class')

%plot of cell death function with fraction of liver for the class of
%delayed liver failure
figure;hold on;
for i=1:length(M_seq_dLF)
plot(Nt,sigfn_dLF(:,i),'color','c','linewidth',2);
end
xlabel('Fraction of liver');xlim([-0.02 1.02]);
ylabel('Cell death function');ylim([-0.02 1.02]);
title('Cell death function for delayed liver failure')
box on;

end
%% ODE Function to solve
function dxdt = LiverRegenODE(t,x,k,x0)
global eps
% Set variable names
G = x(11);epsilon=eps;
Q=max(0,x(1)); P=max(0,x(2)); R=max(0,x(3)); N=(Q+G*(P+R))+epsilon;
IE=x(4); GF=max(1,x(5)); ECM=x(6); TNF=x(7); JAK=x(8); STAT=x(9); SOCS=x(10);
IE0=x0(4); GF0=x0(5); ECM0=x0(6); TNF0=x0(7); JAK0=x0(8); STAT0=x0(9); SOCS0=x0(10);

% Calculate sigmas
sr = 0.5*(1+tanh((k(29)-GF)/k(30))); % sigma_req
sa = 0.5*(1+tanh((k(32)-((N)/k(2)))/k(33))); % sigma_ap

% Calculate reaction rates
r1 = k(24)*(IE-IE0)*Q; % Priming of Quiescent cells
r2 = k(26)*ECM*R; % Replicating cell returning to Quiescence
r3 = k(28)*sr*P; % Requiescence of Primed cells
r4 = k(25)*(GF-GF0)*P; % Primed cells begining Replication
r5 = k(27)*R; % Doubling of Replicating cells
ra1 = k(31)*sa*Q; % cell death of Quiescent cells
ra2 = k(31)*sa*P; % cell death of Primed cells
ra3 = k(31)*sa*R; % cell death of Replicating cells

r6 = k(3)*k(2)/(N); % TNF production by stimulus
r7 = k(5)*TNF/(TNF+k(6)); % TNF production of JAK
r8 = k(4)*TNF; % TNF degradation
r9 = k(7)*JAK; % JAK degredation
r10 = k(9)*JAK*k(8)^2/(k(8)^2+k(10)*(1+SOCS/k(15))); % STAT3 production by JAK
r11 = k(16)*STAT/(STAT+k(17)); % IE production by STAT3
r12 = k(12)*STAT/(STAT+k(13)); % SOCS3 production by STAT3
r13 = k(11)*STAT; % STAT3 degredation
r14 = k(14)*SOCS; % SOCS3 degredation
r15 = k(18)*IE; % IE degredation
r16 = k(19)*TNF*ECM; % ECM degredation by TNF activated MMPs
r17 = k(20)*ECM; % ECM degredation
r18 = k(21)*k(2)/N; % GF production by stimulus
r19 = k(23)*GF*ECM; % GF uptake/production by ECM
r20 = k(22)*GF; % GF degredation

% Set differential equations
dxdt(1) = -r1 + r2 + r3 - ra1; % Q Phase
dxdt(2) = r1 - r4 - r3 - ra2; % P Phase
dxdt(3) = r4 - r2 + r5 - ra3; % R Phase

dxdt(4) = r11 - r15 + k(38); % IE genes
dxdt(5) = r18 - r19 - r20 + k(40); % GF
dxdt(6) = -r16 - r17 + k(39); % ECM

dxdt(7) = r6 - r7 - r8 + k(34); % TNF
dxdt(8) = r7 - r9 + k(35); % JAK
dxdt(9) = r10 - r11 - r12 - r13 + k(36); % STAT3
dxdt(10) = r12 - r14 + k(37); % SOCS3

dxdt(11) = (k(2)/(N))*k(41) - k(2)*k(41); % Cell Mass (M)

dxdt = dxdt';
end
% End of LiverRegenODE
%% generate Sobol sample
function [seq1,seq2,seq3] = Sobolsample(nruns)
p = sobolset(3);
p = scramble(p,'MatousekAffineOwen');
sobol=net(p,nruns);

%creating sample of size nruns for the two parameters
seq1 = sobol(:,1);
seq2 = sobol(:,2);
seq3 = sobol(:,3);

end
%   End of Sobolsample